@@ -25,6 +25,7 @@
 import warnings
 from collections import OrderedDict
 from logging import getLogger
+import hashlib
 
 import numpy as np
 import yaml
@@ -32,6 +33,11 @@ from pyproj import Geod, Proj
 
 from pyresample import _spatial_mp, utils
 
+try:
+    from xarray import DataArray
+except ImportError:
+    DataArray = np.ndarray
+
 logger = getLogger(__name__)
 
 
@@ -63,31 +69,37 @@ class BaseDefinition(object):
         if type(lons) != type(lats):
             raise TypeError('lons and lats must be of same type')
         elif lons is not None:
-            lons = np.asanyarray(lons)
-            lats = np.asanyarray(lats)
+            if not isinstance(lons, (np.ndarray, DataArray)):
+                lons = np.asanyarray(lons)
+                lats = np.asanyarray(lats)
             if lons.shape != lats.shape:
                 raise ValueError('lons and lats must have same shape')
 
         self.nprocs = nprocs
 
         # check the latitutes
-        if lats is not None and ((lats.min() < -90. or lats.max() > +90.)):
-            # throw exception
-            raise ValueError(
-                'Some latitudes are outside the [-90.;+90] validity range')
-        else:
-            self.lats = lats
+        if lats is not None:
+            if isinstance(lats, np.ndarray) and (lats.min() < -90. or lats.max() > 90.):
+                raise ValueError(
+                    'Some latitudes are outside the [-90.;+90] validity range')
+            elif not isinstance(lats, np.ndarray):
+                # assume we have to mask an xarray
+                lats = lats.where((lats >= -90.) & (lats <= 90.))
+        self.lats = lats
 
         # check the longitudes
-        if lons is not None and ((lons.min() < -180. or lons.max() >= +180.)):
-            # issue warning
-            warnings.warn('All geometry objects expect longitudes in the [-180:+180[ range. ' +
-                          'We will now automatically wrap your longitudes into [-180:+180[, and continue. ' +
-                          'To avoid this warning next time, use routine utils.wrap_longitudes().')
-            # wrap longitudes to [-180;+180[
-            self.lons = utils.wrap_longitudes(lons)
-        else:
-            self.lons = lons
+        if lons is not None:
+            if isinstance(lons, np.ndarray) and (lons.min() < -180. or lons.max() >= +180.):
+                # issue warning
+                warnings.warn('All geometry objects expect longitudes in the [-180:+180[ range. ' +
+                              'We will now automatically wrap your longitudes into [-180:+180[, and continue. ' +
+                              'To avoid this warning next time, use routine utils.wrap_longitudes().')
+                # assume we have to mask an xarray
+                # wrap longitudes to [-180;+180[
+                lons = utils.wrap_longitudes(lons)
+            elif not isinstance(lons, np.ndarray):
+                lons = utils.wrap_longitudes(lons)
+        self.lons = lons
 
         self.ndim = None
         self.cartesian_coords = None
@@ -357,8 +369,9 @@ class CoordinateDefinition(BaseDefinition):
     """Base class for geometry definitions defined by lons and lats only"""
 
     def __init__(self, lons, lats, nprocs=1):
-        lons = np.asanyarray(lons)
-        lats = np.asanyarray(lats)
+        if not isinstance(lons, (np.ndarray, DataArray)):
+            lons = np.asanyarray(lons)
+            lats = np.asanyarray(lats)
         super(CoordinateDefinition, self).__init__(lons, lats, nprocs)
         if lons.shape == lats.shape and lons.dtype == lats.dtype:
             self.shape = lons.shape
@@ -429,9 +442,23 @@ class GridDefinition(CoordinateDefinition):
             raise ValueError('2 dimensional lon lat grid expected')
 
 
-class SwathDefinition(CoordinateDefinition):
+def get_array_hashable(arr):
+    """Compute a hashable form of the array `arr`.
 
-    """Swath defined by lons and lats
+    Works with numpy arrays, dask.array.Array, and xarray.DataArray.
+    """
+    # look for precomputed value
+    if isinstance(arr, DataArray) and np.ndarray is not DataArray:
+        return arr.attrs.get('hash', get_array_hashable(arr.data))
+    else:
+        try:
+            return arr.name.encode('utf-8')  # dask array
+        except AttributeError:
+            return arr.view(np.uint8)  # np array
+
+
+class SwathDefinition(CoordinateDefinition):
+    """Swath defined by lons and lats.
 
     Parameters
     ----------
@@ -454,24 +481,49 @@ class SwathDefinition(CoordinateDefinition):
         Swath lats
     cartesian_coords : object
         Swath cartesian coordinates
+
     """
 
     def __init__(self, lons, lats, nprocs=1):
-        lons = np.asanyarray(lons)
-        lats = np.asanyarray(lats)
+        if not isinstance(lons, (np.ndarray, DataArray)):
+            lons = np.asanyarray(lons)
+            lats = np.asanyarray(lats)
         super(SwathDefinition, self).__init__(lons, lats, nprocs)
         if lons.shape != lats.shape:
             raise ValueError('lon and lat arrays must have same shape')
         elif lons.ndim > 2:
             raise ValueError('Only 1 and 2 dimensional swaths are allowed')
 
+        self.hash = None
+
+    def __hash__(self):
+        """Compute the hash of this object."""
+        if self.hash is None:
+            hasher = hashlib.sha1()
+            hasher.update(get_array_hashable(self.lons))
+            hasher.update(get_array_hashable(self.lats))
+            try:
+                if self.lons.mask is not np.bool_(False):
+                    hasher.update(get_array_hashable(self.lons.mask))
+            except AttributeError:
+                pass
+            self.hash = int(hasher.hexdigest(), 16)
+
+        return self.hash
+
     def get_lonlats_dask(self, blocksize=1000, dtype=None):
         """Get the lon lats as a single dask array."""
         import dask.array as da
         lons, lats = self.get_lonlats()
-        lons = da.from_array(lons, chunks=blocksize * lons.ndim)
-        lats = da.from_array(lats, chunks=blocksize * lons.ndim)
-        return da.stack((lons, lats), axis=-1)
+
+        if isinstance(lons.data, da.Array):
+            return lons.data, lats.data
+        else:
+            lons = da.from_array(np.asanyarray(lons),
+                                 chunks=blocksize * lons.ndim)
+            lats = da.from_array(np.asanyarray(lats),
+                                 chunks=blocksize * lats.ndim)
+        return lons, lats
 
     def _compute_omerc_parameters(self, ellipsoid):
         """Compute the oblique mercator projection bouding box parameters."""
@@ -538,13 +590,14 @@ class SwathDefinition(CoordinateDefinition):
 class DynamicAreaDefinition(object):
     """An AreaDefintion containing just a subset of the needed parameters.
 
-    The purpose of this class is to be able to adapt the area extent and size of
-    the area to a given set of longitudes and latitudes, such that e.g. polar
-    satellite granules can be resampled optimaly to a give projection."""
+    The purpose of this class is to be able to adapt the area extent and size
+    of the area to a given set of longitudes and latitudes, such that e.g.
+    polar satellite granules can be resampled optimaly to a give projection.
+    """
 
     def __init__(self, area_id=None, description=None, proj_dict=None,
                  x_size=None, y_size=None, area_extent=None,
-                 optimize_projection=False):
+                 optimize_projection=False, rotation=None):
         """Initialize the DynamicAreaDefinition.
 
         area_id:
@@ -559,6 +612,8 @@ class DynamicAreaDefinition(object):
           The area extent of the area.
         optimize_projection:
           Whether the projection parameters have to be optimized.
+        rotation:
+          Rotation in degrees (negative is cw)
           """
         self.area_id = area_id
         self.description = description
@@ -567,6 +622,7 @@ class DynamicAreaDefinition(object):
         self.y_size = y_size
         self.area_extent = area_extent
         self.optimize_projection = optimize_projection
+        self.rotation = rotation
 
     def compute_domain(self, corners, resolution=None, size=None):
         """Compute size and area_extent from corners and [size or resolution]
@@ -597,7 +653,7 @@ class DynamicAreaDefinition(object):
 
     def freeze(self, lonslats=None,
                resolution=None, size=None,
-               proj_info=None):
+               proj_info=None, rotation=None):
         """Create an AreaDefintion from this area with help of some extra info.
 
         lonlats:
@@ -608,6 +664,8 @@ class DynamicAreaDefinition(object):
           the size of the resulting area.
         proj_info:
           complementing parameters to the projection info.
+        rotation:
+          rotation in degrees (negative is cw)
 
         Resolution and Size parameters are ignored if the instance is created
         with the `optimize_projection` flag set to True.
@@ -632,7 +690,7 @@ class DynamicAreaDefinition(object):
 
         return AreaDefinition(self.area_id, self.description, '',
                               self.proj_dict, self.x_size, self.y_size,
-                              self.area_extent)
+                              self.area_extent, self.rotation)
 
 
 class AreaDefinition(BaseDefinition):
@@ -653,6 +711,8 @@ class AreaDefinition(BaseDefinition):
         x dimension in number of pixels
     y_size : int
         y dimension in number of pixels
+    rotation: float
+        rotation in degrees (negative is cw)
     area_extent : list
         Area extent as a list (LL_x, LL_y, UR_x, UR_y)
     nprocs : int, optional
@@ -676,6 +736,8 @@ class AreaDefinition(BaseDefinition):
         x dimension in number of pixels
     y_size : int
         y dimension in number of pixels
+    rotation: float
+        rotation in degrees (negative is cw)
     shape : tuple
         Corresponding array shape as (rows, cols)
     size : int
@@ -711,7 +773,8 @@ class AreaDefinition(BaseDefinition):
     """
 
     def __init__(self, area_id, name, proj_id, proj_dict, x_size, y_size,
-                 area_extent, nprocs=1, lons=None, lats=None, dtype=np.float64):
+                 area_extent, rotation=None, nprocs=1, lons=None, lats=None,
+                 dtype=np.float64):
         if not isinstance(proj_dict, dict):
             raise TypeError('Wrong type for proj_dict: %s. Expected dict.'
                             % type(proj_dict))
@@ -723,6 +786,10 @@ class AreaDefinition(BaseDefinition):
         self.x_size = int(x_size)
         self.y_size = int(y_size)
         self.shape = (y_size, x_size)
+        try:
+            self.rotation = float(rotation)
+        except TypeError:
+            self.rotation = 0
         if lons is not None:
             if lons.shape != self.shape:
                 raise ValueError('Shape of lon lat grid must match '
@@ -749,8 +816,8 @@ class AreaDefinition(BaseDefinition):
              float(area_extent[3]) -
              float(self.pixel_size_y) / 2)
 
-        # Pixel_offset defines the distance to projection center from origen (UL)
-        # of image in units of pixels.
+        # Pixel_offset defines the distance to projection center from origen
+        # (UL) of image in units of pixels.
         self.pixel_offset_x = -self.area_extent[0] / self.pixel_size_x
         self.pixel_offset_y = self.area_extent[3] / self.pixel_size_y
 
@@ -810,6 +877,7 @@ class AreaDefinition(BaseDefinition):
         fmt += "\tPCS_DEF:\t{proj_str}\n"
         fmt += "\tXSIZE:\t{x_size}\n"
         fmt += "\tYSIZE:\t{y_size}\n"
+        fmt += "\tROTATION:\t{rotation}\n"
         fmt += "\tAREA_EXTENT: {area_extent}\n}};\n"
         area_def_str = fmt.format(name=self.name, area_id=self.area_id,
                                   proj_str=proj_str, x_size=self.x_size,
@@ -823,7 +891,7 @@ class AreaDefinition(BaseDefinition):
         """Test for equality"""
 
         try:
-            return ((self.proj_dict == other.proj_dict) and
+            return ((self.proj_str == other.proj_str) and
                     (self.shape == other.shape) and
                     (np.allclose(self.area_extent, other.area_extent)))
         except AttributeError:
@@ -834,6 +902,13 @@ class AreaDefinition(BaseDefinition):
 
         return not self.__eq__(other)
 
+    def __hash__(self):
+        return hash((
+            self.proj_str,
+            self.shape,
+            self.area_extent
+        ))
+
     def colrow2lonlat(self, cols, rows):
         """
         Return longitudes and latitudes for the given image columns
@@ -1007,6 +1082,13 @@ class AreaDefinition(BaseDefinition):
             Grids of area x- and y-coordinates in projection units
         """
 
+        def do_rotation(xspan, yspan, rot_deg=0):
+            rot_rad = np.radians(rot_deg)
+            rot_mat = np.array([[np.cos(rot_rad),  np.sin(rot_rad)],
+                          [-np.sin(rot_rad), np.cos(rot_rad)]])
+            x, y = np.meshgrid(xspan, yspan)
+            return np.einsum('ji, mni -> jmn', rot_mat, np.dstack([x, y]))
+
         def get_val(val, sub_val, max):
             # Get value with substitution and wrapping
             if val is None:
@@ -1081,9 +1163,15 @@ class AreaDefinition(BaseDefinition):
                 cols = 1
 
         # Calculate coordinates
-        target_x = np.arange(col_start, col_start + cols, dtype=dtype) * self.pixel_size_x + self.pixel_upper_left[0]
-        target_y = np.arange(row_start, row_start + rows, dtype=dtype) * -self.pixel_size_y + self.pixel_upper_left[1]
-        target_x, target_y = np.meshgrid(target_x, target_y)
+        target_x = np.arange(col_start, col_start + cols, dtype=dtype) * \
+            self.pixel_size_x + self.pixel_upper_left[0]
+        target_y = np.arange(row_start, row_start + rows, dtype=dtype) * - \
+            self.pixel_size_y + self.pixel_upper_left[1]
+        if self.rotation != 0:
+            res = do_rotation(target_x, target_y, self.rotation)
+            target_x, target_y = res[0, :, :], res[1, :, :]
+        else:
+            target_x, target_y = np.meshgrid(target_x, target_y)
 
         if is_single_value:
             # Return single values
@@ -1169,7 +1257,7 @@ class AreaDefinition(BaseDefinition):
         res = Array(dsk, name, shape=list(self.shape) + [2],
                     chunks=(blocksize, blocksize, 2),
                     dtype=dtype)
-        return res
+        return res[:, :, 0], res[:, :, 1]
 
     def get_lonlats(self, nprocs=None, data_slice=None, cache=False, dtype=None):
         """Returns lon and lat arrays of area.
@@ -1363,16 +1451,19 @@ class StackedAreaDefinition(BaseDefinition):
     def get_lonlats_dask(self, blocksize=1000, dtype=None):
         """"Return lon and lat dask arrays of the area."""
         import dask.array as da
-        llonslats = []
+        llons = []
+        llats = []
         for definition in self.defs:
-            lonslats = definition.get_lonlats_dask(blocksize=blocksize,
-                                                   dtype=dtype)
+            lons, lats = definition.get_lonlats_dask(blocksize=blocksize,
+                                                     dtype=dtype)
 
-            llonslats.append(lonslats)
+            llons.append(lons)
+            llats.append(lats)
 
-        self.lonlats = da.concatenate(llonslats, axis=0)
+        self.lons = da.concatenate(llons, axis=0)
+        self.lats = da.concatenate(llats, axis=0)
 
-        return self.lonlats
+        return self.lons, self.lats
 
     def squeeze(self):
         """Generate a single AreaDefinition if possible."""
